﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  public interface IGQAPEvaluator : ISingleObjectiveEvaluator {
    ILookupParameter<BoolValue> MaximizationParameter { get; }
    ILookupParameter<DoubleValue> FlowDistanceQualityParameter { get; }
    ILookupParameter<DoubleValue> InstallationQualityParameter { get; }
    ILookupParameter<DoubleValue> OverbookedCapacityParameter { get; }

    double Evaluate(IntegerVector assignment, DoubleMatrix weights, DoubleMatrix distances,
                    DoubleMatrix installCosts, DoubleArray demands, DoubleArray capacities,
                    double transportationCosts, double expectedRandomQuality);

    void Evaluate(IntegerVector assignment, DoubleMatrix weights, DoubleMatrix distances,
                  DoubleMatrix installCosts, DoubleArray demands, DoubleArray capacities,
                  out double flowDistanceQuality, out double installationQuality,
                  out double overbookedCapacity);

    double EvaluateOverbooking(DoubleArray slack, DoubleArray capacities);

    double GetFitness(double flowDistanceQuality, double installationQuality,
                      double overbookedCapacity, double transportationCosts,
                      double expectedRandomQuality);
  }
}
